import React, { Component } from 'react'
import PropTypes from 'prop-types'
import values from 'lodash/values'
import isEqual from 'lodash/isEqual'
import pick from 'lodash/pick'
import noop from 'lodash/noop'

import fbInit from './../../utils/fbInit'

import * as MessengerPluginTypes from './../../constants/MessengerPluginTypes'
import * as MessengerPluginEvents from './../../constants/MessengerPluginEvents'
import * as MessengerPluginSize from './../../constants/MessengerPluginSize'
import * as MessengerPluginColor from './../../constants/MessengerPluginColor'
import * as ClassNames from './../../constants/ClassNames'


export default class FacebookMessengerPlugin extends Component {
  shouldComponentUpdate(props) {
    this.shouldPluginUpdate(props)

    const componentAffectProps = [
      'className', 'pluginClassName',
      'type', 'id', 'appId', 'pageId', 'dataRef', 'size', 'color', 'prechecked',
    ]
    return !isEqual(pick(this.props, componentAffectProps), pick(props, componentAffectProps))
  }

  shouldPluginUpdate(props) {
    const pluginAffectProps = [
      'type', 'appId', 'pageId', 'dataRef', 'size', 'color', 'prechecked',
    ]

    const isPluginAffected = !isEqual(pick(this.props, pluginAffectProps), pick(props, pluginAffectProps))

    if (isPluginAffected) {
      fbInit(props)
    }
  }

  componentDidMount() {
    fbInit(this.props)
    this.subscribe()
  }

  componentWillUnmount() {
    this.unsubscribe()
  }

  subscribe() {
    window.FB.Event.subscribe('messenger_checkbox', this.handleCheckboxEvents.bind(this))
    window.FB.Event.subscribe('send_to_messenger', this.handleButtonEvents.bind(this))
  }

  unsubscribe() {
    window.FB.Event.unsubscribe('messenger_checkbox', this.handleCheckboxEvents.bind(this))
    window.FB.Event.unsubscribe('send_to_messenger', this.handleButtonEvents.bind(this))
  }

  handleCheckboxEvents(event) {
    if (!event.ref || event.ref !== this.props.dataRef) {
      return
    }

    if (event.event === MessengerPluginEvents.CHECKBOX) {
      const checked = event.state == 'checked'
      this.props.onChecked(checked, event)
    }

    if (event.event === MessengerPluginEvents.HIDDEN) {
      this.props.onHide(event)
    }

    if (event.event === MessengerPluginEvents.RENDERED) {
      this.props.onRender(event)
    }
  }

  handleButtonEvents(event) {
    if (!event.ref || event.ref !== this.props.dataRef) {
      return
    }

    if (event.event === MessengerPluginEvents.CLICKED) {
      return this.props.onSubmit(event)
    }

    if (event.event === MessengerPluginEvents.HIDDEN) {
      return this.props.onHide(event)
    }

    if (event.event === MessengerPluginEvents.RENDERED) {
      return this.props.onRender(event)
    }
  }

  getSnippet() {
    const { type, id, appId, pageId, dataRef, size, color, prechecked } = this.props

    if (type === MessengerPluginTypes.CHECKBOX) {
      return (
        `<div
          id='${id}'
          class='${ClassNames.CHECKBOX_PLUGIN}'
          messenger_app_id='${appId}'
          page_id='${pageId}'
          origin='${window.location.origin}'
          data-ref='${dataRef}'
          size=${size}
          prechecked='${JSON.stringify(prechecked)}'
          allow_login='true'></div>`
      )
    }

    if (type === MessengerPluginTypes.BUTTON) {
      return (
        `<div
          id='${id}'
          class='${ClassNames.SEND_TO_MESSENGER_PLUGIN}'
          messenger_app_id='${appId}'
          page_id='${pageId}'
          data-ref='${dataRef}'
          color='${color}'
          size='${size}'></div>`
      )
    }
  }

  render() {
    const { className, pluginClassName } = this.props

    return (
      <div className={className}>
        <div className={pluginClassName} dangerouslySetInnerHTML={{ __html: this.getSnippet() }}></div>
      </div>
    )
  }
}


FacebookMessengerPlugin.propTypes = {
  className: PropTypes.string,
  pluginClassName: PropTypes.string,

  type: PropTypes.oneOf(values(MessengerPluginTypes)),
  id: PropTypes.string.isRequired,
  appId: PropTypes.string.isRequired,
  pageId: PropTypes.string.isRequired,
  dataRef: PropTypes.string.isRequired,
  size: PropTypes.oneOf(values(MessengerPluginSize)),
  color: PropTypes.oneOf(values(MessengerPluginColor)), // button specific
  prechecked: PropTypes.bool, // checkbox specific

  onSubmit: PropTypes.func, // button specific
  onChecked: PropTypes.func, // checkbox specific
  onHide: PropTypes.func,
  onRender: PropTypes.func,
}


FacebookMessengerPlugin.defaultProps = {
  className: '',
  pluginClassName: '',

  type: MessengerPluginTypes.BUTTON,
  size: MessengerPluginSize.LARGE,
  color: MessengerPluginColor.WHITE,
  prechecked: true,

  onSubmit: noop,
  onChecked: noop,
  onHide: noop,
  onRender: noop,
}
